/*
 * A resource offering support for creating processes.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <resource/resource_server.h>

#include "opener_server.h"
#include "process_creating.h"
#include "process_creator_context_resource.h"
#include "process_creator_resource.h"



/* Support for creating processes. */

ProcessCreatorResource::ProcessCreatorResource(const char *rm_filename, file_t *rm_file)
: _creating(rm_filename, rm_file)
{
}

ProcessCreatorResource::~ProcessCreatorResource()
{
}

ipc_server_default_config_type ProcessCreatorResource::config()
{
    return config_Opener;
}



/* ProcessCreator interface methods. */

long ProcessCreatorResource::init_process(l4_cap_idx_t *process)
{
    return _creating.init_process_monitor(process);
}

/* Start the new process, obtaining a reference to it. */

long ProcessCreatorResource::start(int argc, const char *argv[],
                                   l4_cap_idx_t reader, l4_cap_idx_t writer,
                                   l4_cap_idx_t process)
{
    return _creating.start(argc, argv, reader, writer, process);
}



/* Opener interface methods. */

/* Return a context, through which program arguments must be communicated. */

long ProcessCreatorResource::context(l4_cap_idx_t *context)
{
    ProcessCreatorContextResource *resource = new ProcessCreatorContextResource(this);

    /* Complete the initialisation and start a server in a new thread.
       If the thread does not start, the resource will be finalised. */

    return ResourceServer(resource).start_thread(context);
}

// vim: tabstop=4 expandtab shiftwidth=4
